/*
 * Hanashizume no Matsuri for X
 * Copyright (c) 2000, 2001 TF <tf@denpa.org>
 * All rights reserverd.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *      This product includes software developed by TF.
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: resinfo.c,v 1.8 2001/01/30 16:53:25 tf Exp $
 */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "resinfo.h"

/*
 * Predefined Resource Types
 */
#define RT_CURSOR       1
#define RT_BITMAP       2
#define RT_ICON         3
#define RT_MENU         4
#define RT_DIALOG       5
#define RT_STRING       6
#define RT_FONTDIR      7
#define RT_FONT         8
#define RT_ACCELERATOR  9
#define RT_RCDATA       10
#define RT_MESSAGETABLE 11
#define RT_GROUP_CURSOR 12
#define RT_GROUP_ICON   14
#define RT_VERSION      16
#define RT_DLGINCLUDE   17
#define RT_PLUGPLAY     19
#define RT_VXD          20

static void u2aprint(unsigned char *, unsigned short);
static void u2aconv(unsigned char *, unsigned char *, unsigned short);
static void add_resinfo(RESINFO *);

RESINFO resinfo;

unsigned char buf[RES_BUFLEN];

#if 0
int main(int argc, char *argv[])
{
  FILE *fp;
  RESINFO *res_list;

  res_list = create_rlist();
  if (argc != 2) {
    exit(1);
  }
  if ((fp = fopen(argv[1], "rb")) == NULL) {
    exit(1);
  }
  scan_dll(fp, res_list);

  return 0;
}
#endif

int scan_dll(FILE *fp, RESINFO *res_list)
{
    unsigned long i;
    int j, k, l;
    char signature[4];
    unsigned short SecNum;
    unsigned short OptHdSize;
    char SecName[8];
    unsigned long VAddr, PRawData;
    unsigned short TypeNumS, TypeNumI;
    unsigned short NameNumS, NameNumI;
    unsigned short LangNumS, LangNumI;
    unsigned long TypeEntPos, NameEntPos, LangEntPos, DataEntPos, DataPos;
    unsigned long SubDirPos;
    unsigned char Str[RES_BUFLEN];
    unsigned long StrPos;
    unsigned short StrNum;
    int IntID;

    fseek(fp, 0x3c, SEEK_SET);
	fread(&i, 4, 1, fp);
    fseek(fp, i, SEEK_SET);
    fread(signature, 1, 4, fp);
    if (strncmp("PE\0\0", signature, 4) != 0) {
      fprintf(stderr, "This file is not PE format.\n");
        exit(1);
    }
    fseek(fp, 2, SEEK_CUR);
    fread(&SecNum, 2, 1, fp);
    //printf("# of section:%d\n", SecNum);
    fseek(fp, 12, SEEK_CUR);
    fread(&OptHdSize, 2, 1, fp);
    //printf("size of option header:0x%x\n", OptHdSize);
    fseek(fp, i + 4 + 20 + OptHdSize, SEEK_SET);

    for (j = 0; j < SecNum; j++) {
    	fread(SecName, 1, 8, fp);
        if (strncmp(".rsrc\0\0\0", SecName, 8) == 0) {
            fseek(fp, 4, SEEK_CUR);
            fread(&VAddr, 4, 1, fp);
            fseek(fp, 4, SEEK_CUR);
            fread(&PRawData, 4, 1, fp);
            //printf("Virtual address:0x%x\n", VAddr);
            //printf("pointer to RawData:0x%x\n", PRawData);
            break;
        }
        fseek(fp, 32, SEEK_CUR);
    }
    if (j == SecNum) {
        fprintf(stderr, "No .rsrc section.\n");
        exit(1);
    }
    /* See winnt.h for Language */

    fseek(fp, PRawData, SEEK_SET);
    fseek(fp, 12, SEEK_CUR);
    fread(&TypeNumS, 2, 1, fp);
    fread(&TypeNumI, 2, 1, fp);
    //printf("TypeNumS:%d\n", TypeNumS);
    //printf("TypeNumI:%d\n", TypeNumI);
    TypeEntPos = PRawData + 16;
    for (j = 0; j < TypeNumS; j++) {
    	fseek(fp, TypeEntPos + j * 8, SEEK_SET);

        fread(&StrPos, 4, 1, fp);
        //printf("StrPos:0x%x\n", StrPos);
        StrPos &= 0x7fffffff;
        //printf("StrPos:0x%x\n", StrPos);
        fread(&SubDirPos, 4, 1, fp);
        SubDirPos &= 0x7fffffff;
        //printf("PRawData + StrPos:0x%x\n", PRawData + StrPos);
        fseek(fp, PRawData + StrPos, SEEK_SET);
        fread(&StrNum, 2, 1, fp);
        //printf("StrNum:0x%x\n", StrNum);
        fread(Str, StrNum * 2, 1, fp);

        fseek(fp, PRawData + SubDirPos, SEEK_SET);
		fseek(fp, 12, SEEK_CUR);
    	fread(&NameNumS, 2, 1, fp);
    	fread(&NameNumI, 2, 1, fp);
        NameEntPos = PRawData+ SubDirPos + 16;
        for (k = 0; k < NameNumS; k++) {
    		fseek(fp, NameEntPos + k * 8, SEEK_SET);

        	fread(&StrPos, 4, 1, fp);
        	//printf("StrPos:0x%x\n", StrPos);
        	StrPos &= 0x7fffffff;
        	//printf("StrPos:0x%x\n", StrPos);
        	fread(&SubDirPos, 4, 1, fp);
        	SubDirPos &= 0x7fffffff;
            //printf("PRawData + StrPos:0x%x\n", PRawData + StrPos);
        	fseek(fp, PRawData + StrPos, SEEK_SET);
        	fread(&StrNum, 2, 1, fp);
        	//printf("StrNum:0x%x\n", StrNum);
        	fread(Str, StrNum * 2, 1, fp);

        	fseek(fp, PRawData + SubDirPos, SEEK_SET);
			fseek(fp, 12, SEEK_CUR);
    		fread(&LangNumS, 2, 1, fp);
    		fread(&LangNumI, 2, 1, fp);
            LangEntPos = PRawData + SubDirPos + 16;
        	for (l = 0; l < LangNumS; l++) {
    			fseek(fp, LangEntPos + l * 8, SEEK_SET);

        		fread(&StrPos, 4, 1, fp);
        		//printf("StrPos:0x%x\n", StrPos);
        		StrPos &= 0x7fffffff;
        		//printf("StrPos:0x%x\n", StrPos);
        		fread(&DataEntPos, 4, 1, fp);
            	//printf("PRawData + StrPos:0x%x\n", PRawData + StrPos);
        		fseek(fp, PRawData + StrPos, SEEK_SET);
        		fread(&StrNum, 2, 1, fp);
        		//printf("StrNum:0x%x\n", StrNum);
        		fread(Str, StrNum * 2, 1, fp);

        		fseek(fp, PRawData + DataEntPos, SEEK_SET);
                fread(&DataPos, 4, 1, fp);
                DataPos = DataPos + PRawData - VAddr;
   	    	}
    		for (l = 0; l < LangNumI; l++) {
    			fseek(fp, LangEntPos + LangNumS * 8 + l * 8, SEEK_SET);

        		fread(&IntID, 4, 1, fp);

        		fread(&SubDirPos, 4, 1, fp);
        		SubDirPos &= 0x7fffffff;

        		fseek(fp, PRawData + SubDirPos, SEEK_SET);
                fread(&DataPos, 4, 1, fp);
                DataPos = DataPos + PRawData - VAddr;
            }
	    }
    	for (k = 0; k < NameNumI; k++) {
    		fseek(fp, NameEntPos + NameNumS * 8 + k * 8, SEEK_SET);

        	fread(&IntID, 4, 1, fp);

        	fread(&SubDirPos, 4, 1, fp);
        	SubDirPos &= 0x7fffffff;

        	fseek(fp, PRawData + SubDirPos, SEEK_SET);
			fseek(fp, 12, SEEK_CUR);
    		fread(&LangNumS, 2, 1, fp);
    		fread(&LangNumI, 2, 1, fp);
            LangEntPos = PRawData + SubDirPos + 16;
        	for (l = 0; l < LangNumS; l++) {
    			fseek(fp, LangEntPos + l * 8, SEEK_SET);

        		fread(&StrPos, 4, 1, fp);
        		//printf("StrPos:0x%x\n", StrPos);
        		StrPos &= 0x7fffffff;
        		//printf("StrPos:0x%x\n", StrPos);
        		fread(&DataEntPos, 4, 1, fp);
            	//printf("PRawData + StrPos:0x%x\n", PRawData + StrPos);
        		fseek(fp, PRawData + StrPos, SEEK_SET);
        		fread(&StrNum, 2, 1, fp);
        		//printf("StrNum:0x%x\n", StrNum);
        		fread(Str, StrNum * 2, 1, fp);

        		fseek(fp, PRawData + DataEntPos, SEEK_SET);
                fread(&DataPos, 4, 1, fp);
                DataPos = DataPos + PRawData - VAddr;
   	    	}
    		for (l = 0; l < LangNumI; l++) {
    			fseek(fp, LangEntPos + LangNumS * 8 + l * 8, SEEK_SET);

        		fread(&IntID, 4, 1, fp);

        		fread(&SubDirPos, 4, 1, fp);
        		SubDirPos &= 0x7fffffff;

        		fseek(fp, PRawData + SubDirPos, SEEK_SET);
                fread(&DataPos, 4, 1, fp);
                DataPos = DataPos + PRawData - VAddr;
            }
    	}
    }
    for (j = 0; j < TypeNumI; j++) {
    	fseek(fp, TypeEntPos + TypeNumS * 8 + j * 8, SEEK_SET);

        fread(&IntID, 4, 1, fp);
        //printf("IntID:0x%x\n", IntID);
	resinfo.type = IntID;
#if 0
        switch (IntID) {
        case RT_CURSOR:
        	puts("CURSOR"); break;
		case RT_BITMAP:
            puts("BITMAP"); break;
		case RT_ICON:
        	puts("ICON"); break;
        case RT_MENU:
        	puts("MENU"); break;
        case RT_DIALOG:
        	puts("DIALOG"); break;
		case RT_STRING:
        	puts("STRING"); break;
		case RT_FONTDIR:
        	puts("FONTDIR"); break;
		case RT_FONT:
        	puts("FONT"); break;
		case RT_ACCELERATOR:
        	puts("ACCELERATOR"); break;
		case RT_RCDATA:
        	puts("RCDATA"); break;
		case RT_MESSAGETABLE:
        	puts("MESSAGETABLE"); break;
		case RT_GROUP_CURSOR:
        	puts("GROUP_CURSOR"); break;
		case RT_GROUP_ICON:
        	puts("GROUP_ICON"); break;
        case RT_VERSION:
        	puts("VERSION"); break;
        case RT_DLGINCLUDE:
        	puts("DLGINCLUDE"); break;
		case RT_PLUGPLAY:
        	puts("PLUGPLAY"); break;
		case RT_VXD:
        	puts("VXD"); break;
        default:
        	fprintf(stderr, "0x%x", IntID);
        }
        //putchar('\n');
#endif

        fread(&SubDirPos, 4, 1, fp);
        SubDirPos &= 0x7fffffff;

        fseek(fp, PRawData + SubDirPos, SEEK_SET);
		fseek(fp, 12, SEEK_CUR);
    	fread(&NameNumS, 2, 1, fp);
    	fread(&NameNumI, 2, 1, fp);
        //printf("NameNumS:%d\n", NameNumS);
        //printf("NameNumI:%d\n", NameNumI);
        NameEntPos = PRawData+ SubDirPos + 16;
	    for (k = 0; k < NameNumS; k++) {
    		fseek(fp, NameEntPos + k * 8, SEEK_SET);

        	fread(&StrPos, 4, 1, fp);
        	//printf("StrPos:0x%x\n", StrPos);
        	StrPos &= 0x7fffffff;
        	//printf("StrPos:0x%x\n", StrPos);
        	fread(&SubDirPos, 4, 1, fp);
        	SubDirPos &= 0x7fffffff;
            //printf("PRawData + StrPos:0x%x\n", PRawData + StrPos);
        	fseek(fp, PRawData + StrPos, SEEK_SET);
        	fread(&StrNum, 2, 1, fp);
        	//printf("StrNum:0x%x\n", StrNum);
        	fread(Str, StrNum * 2, 1, fp);
            //putchar('\t');
	        u2aconv(Str, buf, StrNum);
		strncpy(resinfo.name, buf, RES_NAMELEN);

        	fseek(fp, PRawData + SubDirPos, SEEK_SET);
			fseek(fp, 12, SEEK_CUR);
    		fread(&LangNumS, 2, 1, fp);
    		fread(&LangNumI, 2, 1, fp);
            LangEntPos = PRawData + SubDirPos + 16;
        	for (l = 0; l < LangNumS; l++) {
    			fseek(fp, LangEntPos + l * 8, SEEK_SET);

        		fread(&StrPos, 4, 1, fp);
        		//printf("StrPos:0x%x\n", StrPos);
        		StrPos &= 0x7fffffff;
        		//printf("StrPos:0x%x\n", StrPos);
        		fread(&DataEntPos, 4, 1, fp);
            	//printf("PRawData + StrPos:0x%x\n", PRawData + StrPos);
        		fseek(fp, PRawData + StrPos, SEEK_SET);
        		fread(&StrNum, 2, 1, fp);
        		//printf("StrNum:0x%x\n", StrNum);
        		fread(Str, StrNum * 2, 1, fp);

        		fseek(fp, PRawData + DataEntPos, SEEK_SET);
                fread(&DataPos, 4, 1, fp);
                DataPos = DataPos + PRawData - VAddr;
   	    	}
    		for (l = 0; l < LangNumI; l++) {
    			fseek(fp, LangEntPos + LangNumS * 8 + l * 8, SEEK_SET);

        		fread(&IntID, 4, 1, fp);
			resinfo.lang = IntID;

        		fread(&SubDirPos, 4, 1, fp);
        		SubDirPos &= 0x7fffffff;

        		fseek(fp, PRawData + SubDirPos, SEEK_SET);
                fread(&DataPos, 4, 1, fp);
                DataPos = DataPos + PRawData - VAddr;
		resinfo.offset = DataPos;
		add_resinfo(res_list);
            }
	    }
    	for (k = 0; k < NameNumI; k++) {
    		fseek(fp, NameEntPos + NameNumS * 8 + k * 8, SEEK_SET);

        	fread(&IntID, 4, 1, fp);
		snprintf(resinfo.name, RES_NAMELEN, "0x%x", IntID);

        	fread(&SubDirPos, 4, 1, fp);
        	SubDirPos &= 0x7fffffff;

        	fseek(fp, PRawData + SubDirPos, SEEK_SET);
			fseek(fp, 12, SEEK_CUR);
    		fread(&LangNumS, 2, 1, fp);
    		fread(&LangNumI, 2, 1, fp);
            LangEntPos = PRawData + SubDirPos + 16;
        	for (l = 0; l < LangNumS; l++) {
    			fseek(fp, LangEntPos + l * 8, SEEK_SET);

        		fread(&StrPos, 4, 1, fp);
        		//printf("StrPos:0x%x\n", StrPos);
        		StrPos &= 0x7fffffff;
        		//printf("StrPos:0x%x\n", StrPos);
        		fread(&DataEntPos, 4, 1, fp);
            	//printf("PRawData + StrPos:0x%x\n", PRawData + StrPos);
        		fseek(fp, PRawData + StrPos, SEEK_SET);
        		fread(&StrNum, 2, 1, fp);
        		//printf("StrNum:0x%x\n", StrNum);
        		fread(Str, StrNum * 2, 1, fp);

        		fseek(fp, PRawData + DataEntPos, SEEK_SET);
                fread(&DataPos, 4, 1, fp);
                DataPos = DataPos + PRawData - VAddr;
   	    	}
    		for (l = 0; l < LangNumI; l++) {
    			fseek(fp, LangEntPos + LangNumS * 8 + l * 8, SEEK_SET);

        		fread(&IntID, 4, 1, fp);
			resinfo.lang = IntID;

        		fread(&SubDirPos, 4, 1, fp);
        		SubDirPos &= 0x7fffffff;

        		fseek(fp, PRawData + SubDirPos, SEEK_SET);
                fread(&DataPos, 4, 1, fp);
                DataPos = DataPos + PRawData - VAddr;
		resinfo.offset = DataPos;
		add_resinfo(res_list);
            }
    	}
    }

    return 0;
}

static void u2aprint(unsigned char *str, unsigned short strnum)
{
	int i;

    for (i = 0; i < strnum; i++) {
    	putchar((int)str[i * 2]);
    }
}

static void u2aconv(unsigned char *src, unsigned char *dst, unsigned short strnum)
{
  int i;

  for (i = 0; i < strnum; i++) {
    dst[i] = (int)src[i * 2];
  }
  dst[i] = '\0';
}

static void add_resinfo(RESINFO *rlist)
{
  RESINFO *tmp;
  RESINFO *p = rlist;

  while(p->next != NULL) {
    p = p->next;
  }

  // printf("%d:%x:%s:%lx\n", resinfo.type, resinfo.lang, resinfo.name, resinfo.offset);

  tmp = (RESINFO *)malloc(sizeof(RESINFO));
  tmp->type = resinfo.type;
  strncpy(tmp->name, resinfo.name, RES_NAMELEN);
  tmp->lang = resinfo.lang;
  tmp->offset = resinfo.offset;
  if (!strcmp(resinfo.name, "CG02444")) {
    /* Maybe archive loader has a bug... */
    tmp->offset += 128;
  }
  tmp->next = NULL;
  
  p->next = tmp;
}

RESINFO *create_rlist(void)
{
  RESINFO *tmp;
  tmp = (RESINFO *)malloc(sizeof(RESINFO));
  tmp->next = NULL;

  return tmp;
}

void free_rlist(RESINFO *rlist)
{
  RESINFO *p = rlist;
  RESINFO *next;

  do {
    next = p->next;
    free(p);
    p = next;
  } while (p != NULL);
}

void print_rlist(RESINFO *rlist)
{
  RESINFO *p = rlist->next; /* first element is dummy */

  do {
    printf("%d:%x:%s:%lx\n", p->type, p->lang, p->name, p->offset);
    p = p->next;
  } while (p != NULL);
}

RESINFO *search_rlist(RESINFO *rlist, const char *key)
{
  RESINFO *p = rlist->next; /* first element should be empty */

  do {
    if (!strcmp(p->name, key)) {
      return p;
    }
    p = p->next;
  } while (p != NULL);

  return NULL;
}
