/*
 * Hanashizume no Matsuri for X
 * Copyright (c) 2000 TF <tf@denpa.org>
 * All rights reserverd.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *      This product includes software developed by TF.
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: store_global.c,v 1.6 2001/01/08 14:13:37 tf Exp $
 */
#include <stdio.h>

#include "hana.h"
#include "flags.h"

static void init_globalinfo(void);
static void print_clearflag(int);

int save_global(const char *filename)
{
  FILE *fp;

  fprintf(stderr, "save global info.\n");

  if ((fp = fopen(filename, "wb")) == NULL) {
    fprintf(stderr, "save_global: Can't open %s\n", filename);
    return -1;
  }

  /* clear flag */
  put4b(fp, scn_stat.clear);

  /* user preference */
  put4b(fp, sys_stat.speed);
  put4b(fp, sys_stat.music_mode);
  put4b(fp, sys_stat.cg_fade);
  put4b(fp, sys_stat.brightness);

  /* seen CG */
  /* seen script */

  fclose(fp);
  return 0;
}

int load_global(const char *filename)
{
  FILE *fp;

  fprintf(stderr, "load global info.\n");

  if ((fp = fopen(filename, "rb")) == NULL) {
    fprintf(stderr, "load_global: Can't open %s\n", filename);
    init_globalinfo();
    return -1;
  }
  
  /* clear flag */
  scn_stat.clear = get4b(fp);
  print_clearflag(scn_stat.clear);

  /* user preference */
  sys_stat.speed = get4b(fp);
  sys_stat.music_mode = get4b(fp);
  sys_stat.cg_fade = get4b(fp);
  sys_stat.brightness = get4b(fp);
  if (sys_stat.brightness != BG_DENSITY1 &&
      sys_stat.brightness != BG_DENSITY2 &&
      sys_stat.brightness != BG_DENSITY3 &&
      sys_stat.brightness != BG_DENSITY4) {
    sys_stat.brightness = BG_DENSITY2;
  }
  
  /* seen CG */
  /* seen script */
  
  fclose(fp);
  return 0;
}

static void init_globalinfo()
{
  scn_stat.clear = 0;
  sys_stat.music_mode = MUSIC_CDDA;
  sys_stat.speed = SPEED_NORMAL;
  sys_stat.cg_fade = False;
}

#define CSTAT(f) ((clear & f) ? "clear" : "yet")

static void print_clearflag(int clear)
{
  fprintf(stderr, "--- Clear status ---\n");
  fprintf(stderr, "Megumi: %s\n", CSTAT(FLAG_CLEAR_MEGUMI));
  fprintf(stderr, "Yue   : %s\n", CSTAT(FLAG_CLEAR_YUE));
  fprintf(stderr, "Ruria : %s\n", CSTAT(FLAG_CLEAR_RURIA));
  fprintf(stderr, "Kurumi: %s\n", CSTAT(FLAG_CLEAR_KURUMI));
  fprintf(stderr, "Minako: %s\n", CSTAT(FLAG_CLEAR_MINAKO));
  fprintf(stderr, "Himiko: %s\n", CSTAT(FLAG_CLEAR_HIMIKO));
  fprintf(stderr, "Hinata: %s\n", CSTAT(FLAG_CLEAR_HINATA));
  fprintf(stderr, "Mao   : %s\n", CSTAT(FLAG_CLEAR_MAO));
  fprintf(stderr, "Sarasa: %s\n", CSTAT(FLAG_CLEAR_SARASA));
  fprintf(stderr, "--------------------\n");
}
